package moca.core.proxy;

import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.List;

import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.message.Message;
import moca.core.proxy.receivers.AdapterManager;
import moca.core.proxy.receivers.CISMessage;
import moca.core.proxy.receivers.CISReceiver;
import moca.core.proxy.receivers.CacheManager;
import moca.core.proxy.receivers.CacheMessage;
import moca.core.proxy.receivers.Dispatcher;
import moca.core.proxy.receivers.Package;
import moca.core.proxy.receivers.Receiver;
import moca.core.proxy.receivers.ReceiverMessage;
import moca.core.proxy.receivers.Sender;
import moca.service.cis.util.DeviceContext;




public class Proxy {
	public static Proxy instance;
	private ClientManager clientManager = ClientManager.getInstance() ;
	private List processors = new ArrayList();
	private Dispatcher disp = new Dispatcher();

	private Proxy() {
		processors.add(disp);
		AdapterManager adaptMan = new AdapterManager();
		processors.add(adaptMan);
		disp.register(AdapterManager.class, adaptMan);
		Sender sender = new Sender();
		disp.register(Sender.class, sender);
		processors.add(sender);
		CacheManager cm = new CacheManager();
		disp.register(CacheManager.class, cm);
		processors.add(cm);
		CISReceiver crec = new CISReceiver();
		disp.register(CISReceiver.class, crec);
		processors.add(crec);
	}

	public static Proxy getInstance() {
		if (instance == null) {
			instance = new Proxy();
		}
		return instance;
	}
	public void addMessage(String clientId, Message msg) {
		Package pack = this.clientManager.validate(clientId, msg);
		if (pack != null) {
			pack.setDestination(CacheManager.class);
			disp.addPackage(pack);
		}
	}
	public void registerClient(ClientInfo clientInfo) {
		clientManager.register(clientInfo);
	}
	public void unregisterClient(String clientId) {
		ClientInfo info = clientManager.unregister(clientId);
		if (info != null) {
			//remova o cache do usuario
			ReceiverMessage msg = new CacheMessage(CacheMessage.CACHE_DESTROY);
			Package pack = new Package(info, msg);
			pack.setDestination(CacheManager.class);
			disp.addPackage(pack);
		}

	}
	public void unregisterAllClients() {
		//TODO Primeiro - Pegar todos os clientes e remover o cache!!

		clientManager.unregisterAll();
	}
	/**
	 * Recupera o identificador do cliente mediante seu endereo de rede
	 * @param clientAddress Endereo do cliente
	 * @return identificador do cliente
	 */
	public String getClientId(InetSocketAddress clientAddress) {
		return clientManager.getClientId(clientAddress);
	}	
	
	public ClientInfo getClientInfo (String id){
		return clientManager.getClientInfo(id);
	}
	/**
	 * Recupera o communication Portocol do cliente mediante seu identificador
	 * @param clientId Identificador do cliente
	 * @return Dados do protocolo de comunicao
	 */
	public CommunicationProtocol getClientProtocol(String clientId) {
		return clientManager.getClientInfo(clientId).getCommunicationProtocol();
	}	
	
	private static class ProxyThread extends Thread {
		private Receiver rec;
		public ProxyThread(Receiver rec) {
			this.rec = rec;
		}
		public void run() {
			this.rec.process();
		}
	}

	public void sendCISMessage(String clientId, int stateIndex, boolean isOn, DeviceContext currentContext) {	
		
		// Atualizar estado
		CISMessage cismsg = new CISMessage(clientId, stateIndex, isOn);
		
		
        // Alterao para incluir o contexto corrente... Lembrando que qdo  um evento da 
		// expresso ALL_DEVICES (tag para sempre executar uma adaptao),
		// o contexto  nulo e no se deve alterar o contexto j armazenado
		Package pack = this.clientManager.validateCIS(clientId, cismsg, currentContext);		
		if (pack != null) {
			pack.setDestination(CISReceiver.class);
			disp.addPackage(pack);
		}
	}

	public void run() {
		int nT;
		for (int i = 0; i < this.processors.size(); i++) {
			Receiver rec = (Receiver) this.processors.get(i);
			if (rec instanceof AdapterManager){
				nT = 20;
			}else if (!(rec instanceof Sender)) {
				nT = 15;
			} else {
				nT = 1;
			}
			for (int j = 0; j < nT; j++) {
				Thread t = new ProxyThread(rec);
				t.setName(rec.getClass().getName());
				t.start();

			}
		}
	}
}
