package moca.core.proxy.actions.listeners;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.actions.Action;
import moca.core.proxy.receivers.Dispatcher;

/**
 * Classe base para os listeners de estado. Listeners so classes que reagem
 * a mudanas no estado de um cliente do proxy. Diferente dos Adapters, que 
 * atuam sobre cada uma das mensagem enviadas a um destinatario, os listeners 
 * de estado agem apenas quando o estado do cliente muda. Esse mecanismo age 
 * de forma independente do envio de mensagem ao cliente.
 * 
 * @author hana
 */
public abstract class StateListener extends Action {
	/**
	 * Este mtodo  executado toda vez que o estado do cliente transiciona de off para on
	 * (o cliente no estava no estado em que se observa e passou a apresentar as caracteristicas do estado)
	 * @param info informaes sobre o cliente
	 * @param disp objeto despachante das mensagens
	 * @throws StateListenerException se algum error ocorrer durante a execuo do listener
	 */
	public abstract void matches(ClientInfo info, Dispatcher disp)
		throws StateListenerException;
	/**
	 * Este mtodo  executado toda vez que o estado do cliente transiciona de on para off
	 * (o cliente estava no estado em que se observa e deixou de apresentar as caracteristicas do estado)
	 * @param info informaes do cliente
	 * @param disp objeto despachante de mensagens
	 * @throws StateListenerException se algum error ocorrer durante a execuo do listener
	 */
	public abstract void unmatches(ClientInfo info, Dispatcher disp)
		throws StateListenerException;
}
