package moca.core.proxy.receivers;

import org.apache.log4j.Logger;
import moca.core.proxy.ClientInfo;
import moca.core.proxy.state.State;

/**
 * Analise as mensagem enviadas pelos listeners do CIS. Este receiver permite
 * que o proxy reaja  mudanas no estado dos clientes atendidos pelo proxy.
 * @author hana
 */
public class CISReceiver extends Receiver {
	/**
	 * Logger para o receiver
	 */
	private static final Logger LOGGER = Logger.getLogger(CISReceiver.class);
	/**
	 * Faz a analise das mensagem provenientes do CIS. 
	 * @param clientInfo identificador do cliente
	 * @param msg mensagem
	 */
	protected ReceiverResult process(ClientInfo clientInfo, ReceiverMessage msg) {
		CISMessage cismsg = (CISMessage) msg;
		State state = clientInfo.getState(cismsg.getStateIndex());
		
		LOGGER.debug("Client " + clientInfo.getClientId() + " State: " + 
				       state.getStateConf().getExpression().getValue() + " set to " + cismsg.isOn());
		boolean lastState;
		lastState = cismsg.isOn()?state.setOn():state.setOff();
		if (lastState != cismsg.isOn()) {
			//se o estado mudou, rode os listeners
			state.runListeners(cismsg.isOn(), disp, clientInfo);
		}
		return null;
	}
}
