package moca.core.proxy.receivers;

import java.io.IOException;
import java.net.SocketException;


import moca.core.proxy.ClientInfo;
import moca.core.proxy.ProxyFramework;
import moca.core.proxy.communication.ClientCommunication;
import moca.core.proxy.communication.CommunicationProtocol;
import moca.core.proxy.message.Message;
import moca.protocol.net.TCPConnection;
import moca.protocol.net.sync.udp.UDPDatagram;

import org.apache.log4j.Logger;


/**
 *
 * Ultimo estagio do pacote dentro do proxy. No Sender, os pacotes processados pelo proxy
 * so encaminhados aos seus respectivos destinatarios, de acordo com seu protocolo de 
 * comunicao.
 * 
 * @author hana
 *
 */
public class Sender extends Receiver {
	
	//static Category 	cat = Category.getInstance(Sender.class.getName());
	private static final Logger cat = Logger.getLogger(Sender.class);

	UDPDatagram udpSend ;
	ProxyFramework proxy;
	
	public Sender(){
		proxy = ProxyFramework.getInstance();
		try {
			udpSend = new UDPDatagram();
		} catch (SocketException e) {
			cat.error(e);
		}		
	}
	
	/**
	* Adiciona um pacote para a lista do Sender
	* @param pack
	*/	
	public void add(Package pack) {
		super.addPackage(pack);
	}
	
	/**
	* Adiciona um pacote para a fila do Sender. Este mtodo faz o tratamento para que
	* a ordem de envio das mensagens servidor-proxy seja mantida dentro da fila do Sender
	*/
	protected final synchronized void addPackage(Package pack) {
		pack.setAsProcessed();
		pack.onSender(this);
	}
	
		
	/* (non-Javadoc)
	 * @see moca.core.proxy.receivers.Receiver#process(moca.core.proxy.ClientInfo, moca.core.proxy.receivers.ReceiverMessage)
	 */
	/**
	 * Processa as mensagem. No caso do Sender o processamento consiste no envio das
	 * mensagens a seus destinatarios
	 * @param info informaes do cliente
	 * @param msg mensagem a ser enviada
	 */
	protected ReceiverResult process(ClientInfo info, ReceiverMessage msg) {		
		
		ClientCommunication clientServer = proxy.getClientCommunication();
		
		//Converter para o tipo Message ou DefaulTt message e enviar ....
		Message message = (Message)msg;
 		
		cat.debug("Message "+ message + " exiting the proxy.");
		
		//envia mensagem s se no for nula
		if (message != null) {
			CommunicationProtocol cm = info.getCommunicationProtocol();
			if ( cm.getCommunicMode().equals(CommunicationProtocol.EVENT)){
				cat.debug("Publish message " + message + " to subject " + message.getSubject() + " to client " + message.getAddressee() + "IP: " + cm.getAddress());
				clientServer.getEventServer().publish(message.getSubject(), (Object)message, cm.getAddress());
			}
			else { //comunicao tcp
				if(cm.getProtocol().equals(CommunicationProtocol.TCP)){	
					// tcp synchronous
					if(cm.getCommunicMode().equals(CommunicationProtocol.SYNC)){
					try {
						cat.debug("Sending TCP message " + message + " to client " + message.getAddressee());
						cm.getSyncTCPConnection().send(message);
					} catch (IOException e) {
						cat.error("Message could not be sent to the client " + info.getClientId() +"." + e);
					}
				}// if
					// tcp asynchronous
					else if(cm.getCommunicMode().equals(CommunicationProtocol.ASYNC)){
						TCPConnection myConnection = null ;
						try {
							ClientInfo clientInfo = proxy.proxy.getClientInfo(((Message)msg).getAddressee()) ;
						//	int port = clientInfo.getCommunicationProtocol().getAddress().getPort() ;    
						//	String host = clientInfo.getCommunicationProtocol().getAsyncTCPConnection().getSocket().getInetAddress().getHostAddress() ; 
							myConnection = new TCPConnection(clientInfo.getCommunicationProtocol().getAddress() ); 
						} catch (IOException e1) {
							cat.warn("Proxy could not send message to client");
						}
						
						try {
							
							myConnection.send((Message)msg) ;
							   
						} catch (IOException e2) {
							System.out.println("> Error when sending a msg to clients.");
						}
						finally{
							try {
								if(myConnection != null)
								    myConnection.close() ;
							} catch (IOException e) {
								;
							}
						}
					}
			}
				else{ //UDP
					try {
						cat.debug("Sending UDP message " + message + " to client " + message.getAddressee());
						byte[] buffer = UDPDatagram.convertObjectToByteArray(message);				 
						udpSend.send(buffer, cm.getAddress());
						
					} catch (IOException e) {
						cat.error("Message could not be sent to the client " + 
								     info.getClientId() +"." + e);
					}
				}
			}		
		}
		return null;
	}
}
