package moca.core.proxy.state;

import java.util.Map;
import org.apache.log4j.Logger;
import moca.core.proxy.actions.Action;

/**
 * Classe que armazena informaes de uma Action, cadastradas no arquivo de configurao
 * do proxy 
 * 
 * @author hana
 *
 */
public class ActionConf {
	private static final Logger LOGGER = Logger.getLogger(ActionConf.class);
	/**
	* Listener
	*/
	public static final int LISTENER = 0;
	/**
	* Adapter
	*/
	public static final int ADAPTER = 1;
	/**
	* Action associado a configuracao 
	*/
	private Action action;
	/**
	 * Tipo do Action (Listener ou Adapter)
	 */
	private int type;
	public ActionConf(String className, Map params) throws Exception {
		try {
			Class cls = Class.forName(className);
			this.action = (Action) cls.newInstance();
			this.action.init(params);
		} catch (Exception e) {
			LOGGER.error("Classe action no pode ser instanciada!", e);
			throw e;
		}
	}
	/**
	 * Tipo do Action
	 * @return retorna o tipo do Action (listener ou adapter)
	 */
	public int getType() {
		return this.type;
	}
	public void setType(int type) {
		this.type = type;
	}
	/**
	 * Retorna o action associado a configuracao
	 * @return action
	 */
	public Action getAction() {
		return this.action;
	}
	/**
	* Compara dois objetos
	*/
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (obj.getClass() != this.getClass()) {
			return false;
		}
		ActionConf conf = (ActionConf) obj;
		if (this.action.equals(conf.action) && this.type == conf.type) {
			return true;
		}
		return false;
	}
}
