package moca.core.proxy.state.filter;

/**
 * Recebe dois valores como operandos e avalia se as saidas so iguais
 * @author hana
 */
public class EqualsOperator extends BinaryOperator implements BooleanValue {
	/**
	* Construtor
	* @param val1 operando 1
	* @param val2 operando 2
	*/
	public EqualsOperator(StringValue val1, StringValue val2) {
		super(val1, val2);
	}
	/**
	 * Faz a avaliao do operador
	 * @param pack dados para avaliaao
	 * @return resultado da avaliao
	 */
	public Object evaluate(MetaPackage pack) {
		String str1 = ((StringValue)this.val1).getStringValue(pack);
		String str2 = ((StringValue)this.val2).getStringValue(pack);
		if (str1 == null || str2 == null) {
			return new Boolean(false);
		}
		//comparacao case insensitive
		return new Boolean(str1.equalsIgnoreCase(str2));
	}
	/**
	 * Valor segundo a interface BooleanValue
	 * @param metapack informacoes para avaliao
	 */
	public boolean getBooleanValue(MetaPackage metapack) {
		Boolean b = (Boolean) this.evaluate(metapack);
		return b.booleanValue();
	}
	/**
	 * Converte a expressao para uma string
	 * @return expressao na forma de string
	 */
	public String toString() {
		return "(" + this.val1 + " = " + this.val2 + ")";
	}

	
	
}
