package moca.core.proxy.state.filter;

import java.lang.reflect.Method;

import moca.core.proxy.ClientInfo;
import moca.core.proxy.message.Message;

import org.apache.log4j.Logger;

/**
 * Esta classe  uma fachada entre o filtro e as informaes disponiveis nos
 * objetos Message e ClientInfo. Isto , ela permite que o filtro recupere, a partir de 
 * um mesmo mtodo, infomaes do cliente e da mensagem sendo avaliadas.
 * 
 * A recuperao dos valores acontece pelo mtodo getAttributeValue()
 * Este mtodo recebe como argumento o nome do atributo cujo valor ser recupeado.
 * Atravs de reflexo um mtodo da forma get<nome do atributo em minusculas>() ser
 * localizado e invocado. O valor de retorno (que deve ser uma string)  ento retonado 
 * pelo mtodo original getAttributeValue().
 * 
 * @author hana
 */
public class MetaPackage {
	/**
	 * Logger para MetaPackage
	 */
	public static final Logger LOGGER = Logger.getLogger(MetaPackage.class);
	/**
	* Informaes do destinatario
	*/
	private ClientInfo clientInfo;
	/**
	* Mensagem
	*/
	private Message msg;
	
	/**
	 * construtor
	 * @param clientInfo informaes do destinatario
	 * @param msg mensagem
	 */
	public MetaPackage(ClientInfo clientInfo, Message msg) {
		this.clientInfo = clientInfo;
		this.msg = msg;
	}
	/**
	 * Modo de comunicao 
	 *  
	 * SYNCHRONOUS comunicao sncrona
	 * ASYNCHRONOUS comunicacao assncrona
	 * @return modo de comunicao
	 */
	private String getcommunicationmode() {
		return this.clientInfo.getCommunicationProtocol().getCommunicMode();
	}
	/**
	 * Protocolo de comunicao - TCP ou UDP
	 * @return protocolo
	 */
	private String getprotocol() {
		return this.clientInfo.getCommunicationProtocol().getProtocol();
	}
	/**
	 * Data type da mensagem - em formato MIME em geral, ex "image/jpeg"
	 * @return tipo da mensagem
	 */
	private String getdatatype() {
		return msg.getDataType();
	}
	/**
	 * identificador do destinatario
	 * @return destinatario
	 */
	private String getclient() {
		return clientInfo.getClientId();
	}
	/**
	 * Assunto da notificao a ser publicada
	 * @return
	 */
	private String getsubject() {
		return msg.getSubject();
	}
	/**
	 * Recupera o valor do campo 'fieldName'
	 * @param fieldName nome do campo (case insensitive)
	 * @return valor do campo
	 */
	public String getAttributeValue(String fieldName) {
		try {
			//localize o mtodo correspondente ao atributo fornecido
			Method m =
				this.getClass().getDeclaredMethod(
					"get" + fieldName.toLowerCase(),
					null);
			//execute o mtodo e recupere seu valor de retorno
			return (String) m.invoke(this, null);
		} catch (Exception e) {
			LOGGER.error("Erro ao recuperar valor para o campo " + fieldName, e);
		} 
		return null;
	}
}
