package moca.core.proxy.state.xml;

import java.util.HashMap;
import java.util.Map;
import moca.core.proxy.state.ActionConf;
import org.apache.log4j.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Faz a leitura de um action
 * @author hana
*/
public class ActionConfXMLReader {
	private static final Logger LOGGER = Logger.getLogger(ActionConfXMLReader.class);
	public static ActionConf read(Element el) {
		//verifique se o n corresponte a um action
		if (!el.getNodeName().equals("Action")) {
			LOGGER.warn("Tag Action esperada. Encontrada: " + el.getNodeName());
			return null;
		}
		//recupere o valor do atributo class 
		String className = el.getAttribute("class");
		//Itere nos ns filhos em busca de possveis parametros
		NodeList nodeList = el.getChildNodes();
		Map paramMap = new HashMap();
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				if (!node.getNodeName().equals("Parameter")) {
					//encontrou um elemento que no  parametro
					LOGGER.warn("Encontrou elemento filho de tag Action que no  Parameter");
					return null;
				}
				el = (Element) node;
				String name = el.getAttribute("name");
				if (name == null || name.trim().equals("")) {
					LOGGER.warn("Action com parametro invalido. Ser descartado.");
				} else {
					String val = ConfiguratorXML.getElementValue(el);
					paramMap.put(name, val);
				}
			}
		}
		ActionConf action;
		try {
			action = new ActionConf(className, paramMap);
		} catch (Exception e) {
			//classe action nao pode ser instanciada
			return null;
		}
		return action;
	}

}
