package moca.core.proxy.state.xml;

import moca.core.proxy.state.ActionConf;
import moca.core.proxy.state.Rule;
import moca.core.proxy.state.filter.Filter;
import moca.core.proxy.state.filter.FilterBuilder;
import moca.core.proxy.state.filter.xml.DOMFilterBuilder;

import org.apache.log4j.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author hana
 */
public class RuleXMLReader {
	private static final Logger LOGGER = Logger.getLogger(RuleXMLReader.class);
	public static Rule read(Element el) {
		if (!el.getNodeName().equals("Rule")) {
			LOGGER.error("Tag Rule esperada. Encontrada: " + el.getNodeName());
			return null;
		}
		String strPriority = el.getAttribute("priority");
		int priority = 0;
		if (strPriority != null && !strPriority.trim().equals("")) {
			//O atributo prioridade  opcional
			priority = Integer.parseInt(strPriority);
			LOGGER.warn("Rule sem prioridade. Ser atribuida prioridade mnima");
		}		
		//itere nos ns filhos da tag Rule
		NodeList nodeList = el.getChildNodes();
		int i;
		Rule rule = null;
		//primeiramente deve ler um filtro
		for (i = 0; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				FilterBuilder fb = new DOMFilterBuilder((Element) node);
				Filter f = fb.createFilter();
				rule = new Rule(f, priority);
				i--;
				break;
			}			
		}
		if (rule == null) {
			//se nenhum filtro foi encontrado, o filtro sera nulo
			rule = new Rule(null, priority);
		}
		//itere os elementos restantes em busca de actions (deve haver ler pelo menos 
		//um action)
		boolean actionWasFound = false;
		for (i++; i < nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			if (node instanceof Element) {
				ActionConf actionConf = ActionConfXMLReader.read((Element)node);
				if (actionConf != null) {
					actionConf.setType(ActionConf.ADAPTER);
					rule.addAction(actionConf);
					actionWasFound = true;
				}
			}
		}
		if (!actionWasFound) {
			LOGGER.error("Rule sem nenhum action!!");
			return null;
		}
		return rule;
	}
}
